#!/usr/local/bin/python2.7

# https://fluxtream.org/dev/#/api-docs

from flask import Flask, abort, request, redirect, send_file, make_response, render_template, Response
from flask.ext.restful import abort, Api, Resource, reqparse, fields, marshal_with
from flask_restful_swagger import swagger
from flask.ext.httpauth import HTTPBasicAuth

app = Flask(__name__)
app.config['SECRET_KEY'] = 'XXXXXXXXXXXXXXXXXXXXXXXX'

auth = HTTPBasicAuth()

import ws.carre as carre
import virtuoso

api = swagger.docs(Api(app), apiVersion='0.1',
                   basePath='http://localhost:5000',
                   resourcePath='/',
                   produces=["application/json", "text/html"],
                   description = 'CARRE web services API',
                   api_spec_url='/api/spec')


@app.errorhandler(404)
def page_not_found(error):
	return "page not found. bye"
	return render_template('page_not_found.html'), 404

@auth.verify_password
def verify_password(username, password):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST
		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sql = "SELECT ?subject ?predicate ?object FROM <https://carre.kmi.open.ac.uk/users/" + username + "> WHERE { ?subject ?predicate ?object } limit 1"
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sparql.setQuery(sql)
		results = sparql.query().convert()
		return True
	except Exception, e:
		return False
	return True

class Measurement(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('name', required=True, type=str, help='Measurement name')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "this provides measurements",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "name",
			"description": "Name of the measurement",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			},
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			measurement = args['name']
			token = args['token']
			rs = carre.getDataContinuous(measurement, token=token)
			return json.loads(rs.to_json(orient='records',date_format='iso'))
		except Exception, e:
			return str(e)

class UserProfile(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "this provides user profile information",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			args = self.reqparse.parse_args()
			token = args['token']
			rs = {}
			rs['username'] = virtuoso.getUsernameFromToken(token)
			if rs['username'] is None:
				return {'error':'invalid token'}
			rs['email'] = virtuoso.getEmailByUsername(rs['username'])
			rs['graphName'] = "https://carre.kmi.open.ac.uk/users/" + rs['username']
			return rs
		except Exception, e:
			return str(e)

class Token(Resource):
	@auth.login_required
	@swagger.operation(
		notes = "this provides authentication tokens. Basic authentication is required.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		import ws.virtuoso as virtuoso
		up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
		username = up.split(':', 1)[0]
		password = up.split(':', 1)[1]

		rs = virtuoso.authenticate(username, password)
		if isinstance(rs, basestring):
			return Response("False")
		d = {}
		d['token'] = rs[0]

		import json
		return Response(json.dumps(d),mimetype="application/json")

class CalendarByToken(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	@swagger.operation(
		notes = "Provides calendar, also accessible through <a href='webcal://carre.kmi.open.ac.uk/ws/icalToken.ics?token=XXX'>webcal://carre.kmi.open.ac.uk/ws/icalToken.ics?token=XXX</a>.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
            "name": "token",
            "description": "Authentication token",
            "required": True,
            "default": "JSON",
            "allowMultiple": False,
            "dataType": 'String',
            "paramType": "query"
          }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import ws.virtuoso as virtuoso
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			token = args['token']
			return Response(carre.createICal(token=token), mimetype='text/calendar')
		except Exception, e:
			return Response(str(e))

class Calendar(Resource):
	@auth.login_required
	@swagger.operation(
		notes = "Provides calendar, also accessible through <a href='webcal://carre.kmi.open.ac.uk/ws/ical.ics'>webcal://carre.kmi.open.ac.uk/ws/ical.ics</a>. Basic authentication is required. ",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import ws.virtuoso as virtuoso
			up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
			username = up.split(':', 1)[0]
			password = up.split(':', 1)[1]
			return Response(carre.createICal(username,password), mimetype='text/calendar')
		except Exception,e:
			return Response(str(e))

class Query(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('sparql', required=True, type=str, help='SPARQL query')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	# @auth.login_required
	@swagger.operation(
	  notes='This is the simplest implementation possible. A SPARQL query is passed to the Virtuoso server and results are returned directly to the client.',
	  responseClass="String in the desired format. JSON (default), XML, TURTLE, N3, RDF",#ModelClass.__name__,
	  nickname='upload',
	  parameters=[
		  {
			"name": "sparql",
			"description": """SPARQL statement to be passed to the RDF repository.""",
								#E.g. SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/gkotsis> WHERE { ?subject ?predicate ?object } limit 1""",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
		  },
          {
            "name": "token",
            "description": "Authentication token",
            "required": True,
            "default": "JSON",
            "allowMultiple": False,
            "dataType": 'String',
            "paramType": "query"
          }

		],
	  responseMessages=[
		  {
			"code": 201,
			"message": "Executed."
		  },
		  {
			"code": 405,
			"message": "Invalid input"
		  }
		]
	  )
	def post(self):
		try:
			self.reqparse.add_argument('sparql', required=True, type=str, help='<sparql> query must be passed')
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			sparql = args['sparql']
			token = args['token']
			rs = virtuoso.executeSPARQL(sparql, token)
			if isinstance(rs, str):
				return {'error':str(type(rs))}
			else:
				return rs

		except Exception, e:
			return str(e)

class User(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('username', required=True, type=str, help='username')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	@swagger.operation(
	  notes='Method for deleting a user',
	  responseClass="Error message if any",#ModelClass.__name__,
	  nickname='upload',
	  parameters=[
		  {
			"name": "username",
			"description": """Username.""",
								#E.g. SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/gkotsis> WHERE { ?subject ?predicate ?object } limit 1""",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
		  },
          {
            "name": "token",
            "description": "Authentication token",
            "required": True,
            "default": "JSON",
            "allowMultiple": False,
            "dataType": 'String',
            "paramType": "query"
          }

		],
	  responseMessages=[
		  {
			"code": 201,
			"message": "Executed."
		  },
		  {
			"code": 405,
			"message": "Invalid input"
		  }
		]
	  )
	def post(self):
		try:
			self.reqparse.add_argument('username', required=True, type=str, help='<username> query must be passed')
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			username = args['username']
			token = args['token']
			return virtuoso.deleteUser(username, token)
		except Exception, e:
			return str(e)

@app.route('/docs')
def docs():
  return redirect('/ws/api/spec.html')



@app.route('/dereference/<path:url>')
@auth.login_required
def dereference(url):
	import carre
	up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
	username = up.split(':', 1)[0]
	password = up.split(':', 1)[1]
	if "http:/carre" in url:
		url = url.replace("http:/carre", "http://carre")
	if "https:/carre" in url:
		url = url.replace("https:/carre", "https://carre")
	return Response(carre.describe(url, username, password))


api.add_resource(Query, '/query')
api.add_resource(User, '/user')
api.add_resource(Measurement, '/measurement')
api.add_resource(UserProfile, '/userProfile')
api.add_resource(Token, '/authenticate')
api.add_resource(Calendar, '/ical.ics')
api.add_resource(CalendarByToken, '/icalToken.ics')


# parser = reqparse.RequestParser()


@app.route('/')
def index():
  return redirect('/ws/api/spec.html')


if __name__ == '__main__':
	app.debug = True
	Query()
	app.run(debug=True, port=65010)
